@@ -2,6 +2,7 @@ import os
 import json
 import functools
 from string import Template
+from typing import Optional
 
 from ...document_chunking.chunking_strategy import ChunkingStrategy, ChunkingSettings
 from ...document_loading import LoadingSettings, LoadingStrategy
@@ -27,8 +28,8 @@ log_result = env_helper.LOG_RESULT
 
 
 class Config:
-    def __init__(self, config: dict):
-        self.prompts = Prompts(config["prompts"])
+    def __init__(self, config: dict, tenant_id: Optional[str] = None):
+        self.prompts = Prompts(config["prompts"], tenant_id=tenant_id)
         self.messages = Messages(config["messages"])
         self.example = Example(config["example"])
         self.logging = Logging(config["logging"])
@@ -122,19 +123,19 @@ class Config:
 
 # TODO: Change to AnsweringChain or something, Prompts is not a good name
 class Prompts:
-    def __init__(self, prompts: dict):
+    def __init__(self, prompts: dict, tenant_id: Optional[str] = None):
         # Try to load prompts from Cosmos DB first, fallback to config
         self.condense_question_prompt = self._get_prompt_from_cosmos_or_config(
-            "condense_question_prompt", prompts["condense_question_prompt"]
+            "condense_question_prompt", prompts["condense_question_prompt"], tenant_id
         )
         self.answering_system_prompt = self._get_prompt_from_cosmos_or_config(
-            "answering_system_prompt", prompts["answering_system_prompt"]
+            "answering_system_prompt", prompts["answering_system_prompt"], tenant_id
         )
         self.answering_user_prompt = self._get_prompt_from_cosmos_or_config(
-            "answering_user_prompt", prompts["answering_user_prompt"]
+            "answering_user_prompt", prompts["answering_user_prompt"], tenant_id
         )
         self.post_answering_prompt = self._get_prompt_from_cosmos_or_config(
-            "post_answering_prompt", prompts["post_answering_prompt"]
+            "post_answering_prompt", prompts["post_answering_prompt"], tenant_id
         )
         self.use_on_your_data_format = prompts["use_on_your_data_format"]
         self.enable_post_answering_prompt = prompts["enable_post_answering_prompt"]
@@ -156,24 +157,31 @@ class Prompts:
             "conversational_flow": self.conversational_flow,
         }
 
-    def _get_prompt_from_cosmos_or_config(self, prompt_name: str, config_value: str) -> str:
+    def _get_prompt_from_cosmos_or_config(self, prompt_name: str, config_value: str, tenant_id: Optional[str] = None) -> str:
         """
         Get prompt from Cosmos DB if available, otherwise use config value.
         
         Args:
             prompt_name: Name of the prompt to retrieve
             config_value: Fallback value from config
+            tenant_id: Optional tenant ID for tenant-specific prompts
             
         Returns:
             Prompt template from Cosmos DB or config fallback
         """
         if prompt_manager:
-            cosmos_prompt = prompt_manager.get_prompt(prompt_name)
+            cosmos_prompt = prompt_manager.get_prompt(prompt_name, tenant_id=tenant_id)
             if cosmos_prompt is not None:
-                logger.info(f"Loaded prompt '{prompt_name}' from Cosmos DB")
+                logger.info(f"Loaded prompt '{prompt_name}' from Cosmos DB for tenant '{tenant_id}'")
                 return cosmos_prompt
             else:
-                logger.info(f"Prompt '{prompt_name}' not found in Cosmos DB, using config fallback")
+                # Fallback to default prompt if tenant-specific prompt not found
+                if tenant_id:
+                    cosmos_prompt = prompt_manager.get_prompt(prompt_name, tenant_id=None)
+                    if cosmos_prompt is not None:
+                        logger.info(f"Loaded default prompt '{prompt_name}' from Cosmos DB")
+                        return cosmos_prompt
+                logger.info(f"Prompt '{prompt_name}' not found in Cosmos DB for tenant '{tenant_id}', using config fallback")
         
         return config_value
 
@@ -289,29 +297,43 @@ class ConfigHelper:
 
     @staticmethod
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=False)
-    def get_active_config_or_default():
-        logger.info("Method get_active_config_or_default started")
-        env_helper = EnvHelper()
-        config = ConfigHelper.get_default_config()
-        if env_helper.LOAD_CONFIG_FROM_BLOB_STORAGE and storage_account:
-            logger.info("Loading configuration from Blob Storage")
-
+    def get_active_config_or_default(tenant_id: Optional[str] = None):
+        """
+        Get the active configuration from blob storage, or the default if it doesn't exist.
+        If a tenant_id is provided, it will first look for a tenant-specific configuration.
+        Falls back to global config if tenant config is not found.
+        """
+        if tenant_id:
             try:
-                # Check if config file exists
-                storage_account.download_blob(CONFIG_FILE_NAME, container_name=CONFIG_CONTAINER_NAME)
-                logger.info("Configuration file found in Blob Storage")
-                default_config = config
-                config_file = storage_account.download_blob(CONFIG_FILE_NAME, container_name=CONFIG_CONTAINER_NAME)
-                config = json.loads(config_file)
-
-                ConfigHelper._set_new_config_properties(config, default_config)
-            except Exception:
-                logger.info(
-                    "Configuration file not found in Blob Storage, using default configuration"
+                config_content = storage_account.read_blob_content(
+                    container_name=CONFIG_CONTAINER_NAME,
+                    blob_name=f"{tenant_id}_{CONFIG_FILE_NAME}",
                 )
+                if config_content:
+                    config = json.loads(config_content)
+                    ConfigHelper._set_new_config_properties(
+                        config, ConfigHelper.get_default_config()
+                    )
+                    return Config(config, tenant_id=tenant_id)
+            except Exception:
+                logger.info(f"No active configuration found for tenant {tenant_id}, falling back to default.")
 
-        logger.info("Method get_active_config_or_default ended")
-        return Config(config)
+        try:
+            config_content = storage_account.read_blob_content(
+                container_name=CONFIG_CONTAINER_NAME,
+                blob_name=CONFIG_FILE_NAME,
+            )
+            if config_content:
+                config = json.loads(config_content)
+                ConfigHelper._set_new_config_properties(
+                    config, ConfigHelper.get_default_config()
+                )
+                # Pass tenant_id even to default config for prompt resolution
+                return Config(config, tenant_id=tenant_id)
+        except Exception:
+            logger.info("No active configuration found, falling back to default.")
+
+        return Config(ConfigHelper.get_default_config(), tenant_id=tenant_id)
 
     @staticmethod
     @functools.cache
@@ -322,16 +344,30 @@ class ConfigHelper:
 
     @staticmethod
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=log_result)
-    def save_config_as_active(config):
-        ConfigHelper.validate_config(config)
-        if storage_account:
-            storage_account.upload_blob(
-                blob_name=CONFIG_FILE_NAME,
-                data=json.dumps(config, indent=2),
-                container_name=CONFIG_CONTAINER_NAME,
-                overwrite=True,
-                content_type="application/json",
-            )
+    def save_config_as_active(config, tenant_id: Optional[str] = None):
+        """
+        Saves the given configuration as the active one in blob storage.
+        If a tenant_id is provided, saves a tenant-specific configuration.
+        Also saves prompts to Cosmos DB using tenant_id for isolation.
+        """
+        config_name = f"{tenant_id}_{CONFIG_FILE_NAME}" if tenant_id else CONFIG_FILE_NAME
+        
+        storage_account.upload_blob(
+            container_name=CONFIG_CONTAINER_NAME,
+            blob_name=config_name,
+            data=json.dumps(config, indent=4),
+            overwrite=True,
+        )
+
+        # Also save prompts to Cosmos DB
+        if prompt_manager:
+            prompts_to_save = []
+            for key, value in config["prompts"].items():
+                # Only save actual prompt strings
+                if "prompt" in key:
+                    prompts_to_save.append({"prompt_name": key, "prompt_data": value})
+            if prompts_to_save:
+                prompt_manager.save_prompts_batch(prompts_to_save, tenant_id=tenant_id)
 
     @staticmethod
     @logger.trace_function(log_execution=log_execution, log_args=False, log_result=log_result)
